/* PktScrpt.h
 *
 * Include for all parts of the translator.
 *
 * Copyright 1993 Kenneth Rice
 *
 * You may freely distribute this source code, and use all or part of
 * this source code in all software including commercial, freeware,
 * shareware and private applications.
 *
 * Please report bugs, fixes, etc to me at:
 *   kenr@xis.xerox.com (internet)
 * or
 *   73577,1653 (compuserve)
 *
 * Created	05/31/93
 * Revised  01/15/98 by Brian Barnt (Added Print function)
 * Revised  07/29/98 by Brian Barnt (Cleaned up comments)
 */


#ifndef _H_PktScrpt_
#define _H_PktScrpt_



#define SUCCESS				0
#define FAILED 				(-1)
#define FAILED_SYNTAX		(-2)
#define FAILED_VALUE		   (-3)
#define FAILED_MEMORY		(-4)
#define FAILED_MAKEPACKET	(-5)

#define BUFSZ           81

#define NIL					0

#define TRUE				1
#define FALSE				0

/* Output Types */

#define OT_SERIAL 0
#define OT_SENDER 1

typedef enum
{
	tokInvalid = 0,		/* Marker for invalid token. */
	tokEOF,
	tokRepeat,
	tokTimes,
	tokEndRepeat,
	tokSend,
	tokRaw,
	tokXOR,
	tokBaseline,
	tokOn,
	tokOff,
	tokForward,
	tokReverse,
	tokStop,
	tokEmergencyStop,
   tokPrint,                        /* BJB Print Command */
   tokString,                       /* BJB String Data   */
	tokNumber
} TokenType;

typedef void	*EmitterTagType;	   /* for tagging other end of emits
										       * that have intervening emits.
										       */
typedef enum
{
	dataElem = 1,
	resetCountElem,
	repeatElem,
   dataStringElem                     /* BJB data element for strings */
} CodeElemType;


typedef struct CodeElement
{
	CodeElemType			type;
	struct CodeElement		*pNext;		/* ptr to next statement. */
	
	/* Used only for dataElem. */
	unsigned char			*pData;
	long					   dataSize;
	
	/* Used only for repeatElem. */
	long					repeatCount;
	long					actualCount;	/* used at run time. */
	struct CodeElement		*pBody;		/* ptr to statements inside loop. */

} CodeElement;


/* Sender.c  - Non-Portable */ 

void                 clr_under(void);
int                  init_send(void);
int                  set_clk(unsigned short  iclk0t,unsigned short  iclk0h,unsigned short  iclk1t);
int                  start_clk(void);
int                  stop_clk(void);
int                  send_bytes(unsigned int  icnt,unsigned char  ibyte);
int                  send_stretched_byte(unsigned short  iclk0t,unsigned short  iclk0h,unsigned char  ibyte);
int                  send_pkt(unsigned const char  *ibytes, unsigned long lsize);
int                  errprint_ok(void);
void                 print_stats(void);
int                  set_pc_delay_1usec(void);
unsigned int         pc_delay_high(unsigned int  idelay);
unsigned int         pc_delay_low(unsigned int  idelay);

/* Async.c - Non-Portable */ 
void	               SendBytes(unsigned char *pBuf, long bytesInBuf,void * pAppData, void * pSenderData, char OutputType);

/* Emmitter.c */ 
void                 ResetEmitter(void);
struct CodeElement   *GetCode(void);
void                 FreeCode(struct  CodeElement *pCode);
long                 CountTotalBytesToSend(struct  CodeElement *pCode);
void                 EmitRaw(unsigned char  *pBuf,long  numBytes);
void                 EmitPacket(unsigned char  *pBuf,long  numBytes,char  OutputType);
EmitterTagType       EmitRepeat(long  repeatCount);
void                 EmitEndRepeat(void  *tag);

/* Run.c (Interpreter.c) */ 
void                 RunCode(struct  CodeElement *pCode,char  OutputType);

/* Lexer.c */ 
void                 ResetLexer(const char  *pTextParm,long  textSizeParm);
void                 GetErrorContext(long  *pLineNo,long  *pPosInLine,char  *pLineText,long  maxLineText);
TokenType            GetToken(long  *pValue, char *pString);

/* Parser.c */ 
int                  ParseScript(const char  *pTextParm,long  textSizeParm,char  OutputType);
void                 ThrowErr(int  code);

/* Symbol.c */ 
int                  LookupSymbol(char  *pStr);
int                  InsertSymbol(char  *pStr,int  tok);

/* Makepckt.c */ 
int                  MakePacketVA(unsigned char  *pSerialBuf,long  *pSerialBufCnt,long  maxSerialBytes,char  OutputType,...);
int                  MakePacket(unsigned char  *pSerialBuf,long  *pSerialBufCnt,long  maxSerialBytes,char  OutputType,unsigned char  *pBytes,long  numBytes);

/* Main.c - Non-Portable */ 
void                 ReportParseError(int  errCode,long  errLine,char  *pErrLineText,long  errPos);
int                  atoh(char  *s);
void                 CalcDivisor(char  *s);
void                 CalcOutput(char  *s);
void                 CalcPortAddress(char  *s);
void                 ReadConfigFile(void);
int                  main(int  iArgCnt,char  * *pszArg);
long                 flength(struct  _iobuf *fp);


#endif /* _H_PktScrpt_ */
