/* PortAsnc.h
 *
 * Portable part of the interface for AsynchSend.c
 *
 * Copyright 1993 Kenneth Rice
 *
 * You may freely distribute this source code, and use all or part of
 * this source code in all software including commercial, freeware,
 * shareware and private applications.
 *
 * Please report bugs, fixes, etc to me at
 *   kenr@xis.xerox.com
 * or
 *   73577,1653 (compuserve)
 *
 * Created	03/05/93
 *			06/27/93	Generalized for use in any program.
 *       22-Nov-96 Revised to allow for Serial/Sender Board Output by BJB
 */


#ifndef _H_PortAsnc_
#define _H_PortAsnc_


typedef void		*AsynchDataPtr;


/* Supplied by AsynchSend.c (non-portable).
 * Sends the specified bytes out the serial port. May start off the asynchronous
 * running if it is the first things sent in a session.
 * The value passed in pAppData is passed back to the SendComplete routine when
 * the specified bytes have been completely sent.
 * The pSenderData should be set to NIL for the first call. When called from the
 * completion routine, pSenderData should be set the the same value as the sender
 * data passed to the completion routine.
 *
 * NOTE: This routine may return long before the data is finished being sent.
 *       When the data is finished being sent, SendComplete will be called.
 *       Do not call this routine again until either an error occurs or SendComplete
 *       is called.
 */
void	SendBytes(unsigned char *pBuf, long bytesInBuf,
									AsynchDataPtr pAppData, AsynchDataPtr pSenderData, char OutputType);


/* Supplied by the app (portable).
 * Called by the asynch send stuff when the bytes passed to SendBytes are
 * finished being sent. pAppData is the same value passed to SendBytes by the
 * application, and pSenderData is the value that must be passed as sender data
 * to any call to SendBytes made from this completion routine.
 *
 * NOTE: This completion routine is called at interupt time. It may not have access
 *       global variables, and it should not call any routines that are not safe
 *       to call at interupt time.
 */
void SendComplete(AsynchDataPtr pAppData, AsynchDataPtr pSenderData, char OutputType);

#endif /* _H_PortAsnc_ */


