/* Interpreter.c
 *
 * Object code interpreter for packet script program.
 *
 * Copyright 1993 Kenneth Rice
 *
 * You may freely distribute this source code, and use all or part of
 * this source code in all software including commercial, freeware,
 * shareware and private applications.
 *
 * Please report bugs, fixes, etc to me at:
 *   kenr@xis.xerox.com (internet)
 * or
 *   73577,1653 (compuserve)
 *
 * Created	06/27/93
 * Revised  05/30/96 by Brian Barnt (Recompiled for MS-DOS & Sender Boards)
 *                   revisions are noted by _BJB_, and old code is commented
 *                   out.
 * Revised  01/17/98 by Brian Barnt (Added Print function)
 * Revised  07/29/98 by Brian Barnt (Cleaned up comments)
 */

#include <stdio.h>   // BJB
#include "PortAsnc.h"
#include "PktScrpt.h"


typedef struct
{
	CodeElement		*pNextInstruction;
} AppDataRec;

AppDataRec	ourData;

									
/* RunCode
 *
 * Run the specified object code asynchronously.
 */
void RunCode(CodeElement *pCode, char OutputType)
{
	   ourData.pNextInstruction = pCode;
	   SendComplete((AsynchDataPtr) &ourData, NIL, OutputType);
}


/* SendComplete
 *
 * Do the next instruction in the program. Actually, does all the instructions up to the
 * next one that actually sends data, at which point it sends the data asynchronously
 * and returns.
 *
 * Called as the completion routine for the asynch stuff.
 */
void SendComplete(AsynchDataPtr pAppData, AsynchDataPtr pSenderData, char OutputType)
{
	CodeElement		*pCur;
	int				resetCount = FALSE;
	AppDataRec		*pData = (AppDataRec *) pAppData;

	while (pData->pNextInstruction != NIL)
	{
		switch (pData->pNextInstruction->type)
		{
		case dataElem:
			pCur = pData->pNextInstruction;
			pData->pNextInstruction = pData->pNextInstruction->pNext;

			SendBytes(pCur->pData, pCur->dataSize, pAppData, pSenderData, OutputType);

// BJB: To execute this code sychronusly under MS-DOS, the return must be
//      commented out, and the break must be added.
//      If/When this is ported to Windows, the return must be reinserted, and
//      SendComplete must be called by the driver when the transmitter
//      buffer is empty. (Like the original Mac program)
//			return;

         break;
		
		case resetCountElem:
			resetCount = TRUE;
			pData->pNextInstruction = pData->pNextInstruction->pNext;
			break;
		
		case repeatElem:
			if (resetCount)
			{
				resetCount = FALSE;
				pData->pNextInstruction->actualCount = pData->pNextInstruction->repeatCount;
			}
			if (pData->pNextInstruction->actualCount > 0)
			{
				pData->pNextInstruction->actualCount -= 1;
				pData->pNextInstruction = pData->pNextInstruction->pBody;
			}
			else
				pData->pNextInstruction = pData->pNextInstruction->pNext;
			break;
// BJB
      case dataStringElem:
			pCur = pData->pNextInstruction;
			PrintMessage(pCur->pData);
			pData->pNextInstruction = pData->pNextInstruction->pNext;
			break;
		}
	}
}
